import { NavigationE2EComponent } from 'apps/alfa-e2e/src/components/navigation/navigation.e2e.component';
import { VorgangListE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-list.e2e.component';
import { VorgangSearchE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-search.e2e.component';
import { VorgangViewsE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-views.e2e.component';
import { HeaderE2EComponent } from 'apps/alfa-e2e/src/page-objects/header.po';
import { MainPage, waitForSpinnerToDisappear } from 'apps/alfa-e2e/src/page-objects/main.po';
import { isKeyboardFocused, isOdsFocused } from 'apps/alfa-e2e/src/support/angular.util';
import { dropCollections, pressTab } from 'apps/alfa-e2e/src/support/cypress-helper';
import { exist, haveFocus } from 'apps/alfa-e2e/src/support/cypress.util';
import { initUsermanagerUsers, loginAsSabine } from 'apps/alfa-e2e/src/support/user-util';

describe('VorgangList Page', () => {
  const mainPage: MainPage = new MainPage();
  const header: HeaderE2EComponent = mainPage.getHeader();
  const vorgangSearch: VorgangSearchE2EComponent = header.getVorgangSearch();

  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
  const views: VorgangViewsE2EComponent = mainPage.getViews();

  const navigation: NavigationE2EComponent = mainPage.getNavigation();

  before(() => {
    initUsermanagerUsers();

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('initial', () => {
    it('should select header logo as start element', () => {
      header.getLogo().click();

      //TODO Pruefung ergaenzen
    });
  });

  describe('Tab through', () => {
    describe('header', () => {
      it('should focus search button', () => {
        pressTab();

        isKeyboardFocused(vorgangSearch.getSearchButton());
      });

      it('should focus search field', () => {
        pressTab();

        haveFocus(vorgangSearch.getInput());
      });

      it('should focus help menu icon', () => {
        pressTab();

        isKeyboardFocused(header.getHelpMenu().getButton());
      });

      it('should focus settings icon', () => {
        pressTab();

        isOdsFocused(header.getUserSettings().getButton());
      });

      it('should focus user icon', () => {
        pressTab();

        isKeyboardFocused(header.getCurrentUserProfile().getUserIconButton());
      });
    });

    describe('subnavigation', () => {
      it('should focus Meine filter', () => {
        pressTab();

        isKeyboardFocused(navigation.getMeineVorgaengeFilter());
      });

      it('should focus Alle filter', () => {
        pressTab();

        isKeyboardFocused(navigation.getAlleFilter());
      });

      it('should focus Nicht zugewiesen filter', () => {
        pressTab();

        isKeyboardFocused(navigation.getNichtZugewiesenFilter());
      });
    });

    describe('views', () => {
      it('should focus Neu view', () => {
        pressTab();

        isKeyboardFocused(views.getNeu().getButton());
      });

      it('should focus Angenommen view', () => {
        pressTab();

        isKeyboardFocused(views.getAngenommen().getButton());
      });

      it('should focus InBearbeitung view', () => {
        pressTab();

        isKeyboardFocused(views.getInBearbeitung().getButton());
      });

      it('should focus Beschieden view', () => {
        pressTab();

        isKeyboardFocused(views.getBeschieden().getButton());
      });

      it('should focus Abgeschlossen view', () => {
        pressTab();

        isKeyboardFocused(views.getAbgeschlossen().getButton());
      });

      it('should focus Verworfen view', () => {
        pressTab();

        isKeyboardFocused(views.getVerworfen().getButton());
      });

      it('should focus Zu Löschen view', () => {
        pressTab();

        isKeyboardFocused(views.getZuLoeschen().getButton());
      });

      it('should focus Wiedervorlagen view', () => {
        pressTab();

        isKeyboardFocused(views.getWiedervorlagen().getButton());
      });

      it('should focus VorgangList view', () => {
        pressTab();

        isKeyboardFocused(views.getVorgangList().getButton());
      });

      it('should focus Suche view', () => {
        pressTab();

        isKeyboardFocused(views.getSuche().getButton());
      });
    });
  });
});
