import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { AfterViewInit, Component, ElementRef, Input, ViewChild } from '@angular/core';
import { Observable, of, tap } from 'rxjs';
import { Organisationseinheit } from '../../../user/user.model';
import { OrganisationseinheitFormService } from './organisationseinheit-form.service';

@Component({
  selector: 'admin-organisationseinheit-form',
  templateUrl: './organisationseinheit-form.component.html',
  providers: [OrganisationseinheitFormService],
})
export class OrganisationseinheitFormComponent implements AfterViewInit {
  @Input() organisationseinheitItems: StateResource<Organisationseinheit[]> = createStateResource([]);

  static CREATE_LABEL: string = 'Neue Organisationseinheit anlegen';
  static EDIT_LABEL: string = 'Organisationseinheit bearbeiten';

  protected readonly OrganisationseinheitFormService = OrganisationseinheitFormService;

  @ViewChild('OrganisationseinheitDialog') private dialogRef: ElementRef<HTMLDialogElement>;
  dialog: HTMLDialogElement;

  submitInProgress$: Observable<boolean> = of(false);

  label: string;

  constructor(public formService: OrganisationseinheitFormService) {}

  ngAfterViewInit(): void {
    this.dialog = this.dialogRef.nativeElement;
  }

  public submit() {
    this.submitInProgress$ = this.formService.submit().pipe(tap((progress: boolean) => this.handleProgressChange(progress)));
  }

  handleProgressChange(progress: boolean): void {
    if (!progress) {
      this.completeIfNoErrors();
    }
  }

  completeIfNoErrors(): void {
    if (!this.formService.isInvalid()) {
      this.complete();
    }
  }

  public open(): void {
    this.label = OrganisationseinheitFormComponent.CREATE_LABEL;
    this.formService.reset();
    this.dialog.showModal();
  }

  public openEdit(organisationseinheit: Organisationseinheit): void {
    this.label = OrganisationseinheitFormComponent.EDIT_LABEL;
    this.formService.patch(organisationseinheit);
    this.dialog.showModal();
  }

  complete(): void {
    this.dialog.close();
    this.formService.reset();
  }
}
