import {
  ConvertForDataTestPipe,
  StateResource,
  convertForDataTest,
  createStateResource,
} from '@alfa-client/tech-shared';
import {
  dispatchEventFromFixture,
  existsAsHtmlElement,
  getElementFromFixture,
  notExistsAsHtmlElement,
} from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { createOrganisationseinheit } from '../../../../../test/user/user';
import { MoreItemButtonComponent } from '../../../shared/more-menu/more-item-button/more-item-button.component';
import { MoreMenuComponent } from '../../../shared/more-menu/more-menu.component';
import { Organisationseinheit } from '../../../user/user.model';
import { OrganisationseinheitListComponent } from './organisationseinheit-list.component';

describe('OrganisationseinheitListComponent', () => {
  let component: OrganisationseinheitListComponent;
  let fixture: ComponentFixture<OrganisationseinheitListComponent>;

  const emptyMessageSelector: string = getDataTestIdOf('organisationseinheit-empty-message');
  const tableSelector: string = getDataTestIdOf('organisationseinheit-table');

  const organisationseinheitElementSelector = (
    organisationseinheit: Organisationseinheit,
    cell: string,
  ) =>
    getDataTestIdOf(convertForDataTest(`organisationseinheit-${cell}-${organisationseinheit.id}`));

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        OrganisationseinheitListComponent,
        ConvertForDataTestPipe,
        MockComponent(MoreMenuComponent),
        MockComponent(MoreItemButtonComponent),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(OrganisationseinheitListComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('table rows', () => {
    describe('without organisationseinheit items', () => {
      it('should show empty message', () => {
        existsAsHtmlElement(fixture, emptyMessageSelector);
      });
      it('should not show table', () => {
        notExistsAsHtmlElement(fixture, tableSelector);
      });
    });

    describe('with organisationseinheit items', () => {
      const organisationseinheitItems: StateResource<Organisationseinheit[]> = createStateResource([
        createOrganisationseinheit(),
        createOrganisationseinheit(),
      ]);
      beforeEach(() => {
        component.organisationseinheitItems = organisationseinheitItems;
        fixture.detectChanges();
      });

      it('should show table', () => {
        existsAsHtmlElement(fixture, tableSelector);
      });
      it('should not show empty message', () => {
        notExistsAsHtmlElement(fixture, emptyMessageSelector);
      });

      it('should show rows in order', () => {
        const tableElement: HTMLTableElement = getElementFromFixture(fixture, tableSelector);
        const rows: HTMLTableRowElement[] = Array.from(tableElement.querySelectorAll('tr[id]'));
        const rowIds: string[] = rows.map((row) => row.id);

        expect(rowIds).toEqual(
          organisationseinheitItems.resource.map(
            (organisationseinheit: Organisationseinheit) => organisationseinheit.id,
          ),
        );
      });

      it.each(organisationseinheitItems.resource)(
        'should show name of organisationseinheit %#',
        (organisationseinheit: Organisationseinheit) => {
          const nameTableCell = getElementFromFixture(
            fixture,
            organisationseinheitElementSelector(organisationseinheit, 'name'),
          );

          expect(nameTableCell.textContent.trim()).toBe(organisationseinheit.name);
        },
      );

      it.each(organisationseinheitItems.resource)(
        'should show organisationseinheitId of organisationseinheit %#',
        (organisationseinheit: Organisationseinheit) => {
          const attrTableCell = getElementFromFixture(
            fixture,
            organisationseinheitElementSelector(organisationseinheit, 'attr'),
          );

          expect(attrTableCell.textContent.trim()).toBe(
            `OrganisationseinheitID: ${organisationseinheit.organisationseinheitIds.join(', ')}`,
          );
        },
      );

      it.each(organisationseinheitItems.resource)(
        'should emit editOrganisationseinheit %# on edit button click ',
        (organisationseinheit: Organisationseinheit) => {
          component.editOrganisationseinheit.emit = jest.fn();

          dispatchEventFromFixture(
            fixture,
            organisationseinheitElementSelector(organisationseinheit, 'edit'),
            'clickEmitter',
          );

          expect(component.editOrganisationseinheit.emit).toHaveBeenCalledWith(
            organisationseinheit,
          );
        },
      );

      it.each(organisationseinheitItems.resource)(
        'should emit deleteOrganisationseinheit %# on delete button click ',
        (organisationseinheit: Organisationseinheit) => {
          component.deleteOrganisationseinheit.emit = jest.fn();

          dispatchEventFromFixture(
            fixture,
            organisationseinheitElementSelector(organisationseinheit, 'delete'),
            'clickEmitter',
          );

          expect(component.deleteOrganisationseinheit.emit).toHaveBeenCalledWith(
            organisationseinheit,
          );
        },
      );
    });
  });
});
