import { ConvertForDataTestPipe, InvalidParam } from '@alfa-client/tech-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule } from '@angular/forms';
import { createInvalidParam, createProblemDetail } from 'libs/tech-shared/test/error';
import { TextFieldComponent } from './text-field.component';

import * as TechValidationUtil from 'libs/tech-shared/src/lib/validation/tech.validation.util';

describe('TextFieldComponent', () => {
  let component: TextFieldComponent;
  let fixture: ComponentFixture<TextFieldComponent>;

  const label = 'custom';

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [TextFieldComponent, ConvertForDataTestPipe],
      imports: [ReactiveFormsModule],
    }).compileComponents();

    fixture = TestBed.createComponent(TextFieldComponent);
    component = fixture.componentInstance;
    component.label = label;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('getErrorMessage', () => {
    it('should call getMessageForInvalidParam()', () => {
      const getMessageForInvalidParam: jest.SpyInstance<string, [string, InvalidParam]> =
        jest.spyOn(TechValidationUtil, 'getMessageForInvalidParam');
      const invalidParam: InvalidParam = createInvalidParam();

      component.getErrorMessage(invalidParam);

      expect(getMessageForInvalidParam).toHaveBeenCalledWith(label, invalidParam);
    });
  });

  describe('show error messages', () => {
    it('should not call getErrorMessage() if no error', () => {
      component.getErrorMessage = jest.fn();
      component.fieldControl.setErrors({});

      fixture.detectChanges();

      expect(component.getErrorMessage).not.toHaveBeenCalled();
    });

    it('should call getErrorMessage() if error', () => {
      component.getErrorMessage = jest.fn();

      component.fieldControl.setErrors({
        ...createProblemDetail(),
        invalidParams: [{ ...createInvalidParam(), name: 'settingBody.absender.name' }],
      });

      fixture.detectChanges();

      expect(component.getErrorMessage).toHaveBeenCalled();
    });
  });
});
