import { TestBed } from '@angular/core/testing';
import { ENVIRONMENT_CONFIG, Environment } from '@alfa-client/environment-shared';
import { mock } from '@alfa-client/test-utils';
import { OAuthService } from 'angular-oauth2-oidc';
import { createEnvironment } from 'libs/environment-shared/test/environment';
import { AuthService } from './auth.service';

import * as Storage from './storage/storage';

describe('AuthService', () => {
  let service: AuthService;
  const oAuthService = mock(OAuthService);
  const environment: Environment = createEnvironment();

  beforeEach(() => {
    TestBed.configureTestingModule({
      providers: [
        {
          provide: OAuthService,
          useValue: oAuthService,
        },
        {
          provide: ENVIRONMENT_CONFIG,
          useValue: environment,
        },
      ],
    });
    service = TestBed.inject(AuthService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('hasNoValidToken', () => {
    it('should return false if both tokens are valid', () => {
      jest.spyOn(oAuthService, 'hasValidAccessToken').mockReturnValue(true);
      jest.spyOn(oAuthService, 'hasValidIdToken').mockReturnValue(true);

      TestBed.runInInjectionContext(() => {
        const result: boolean = service.hasNoValidToken();

        expect(result).toBeFalsy();
      });
    });

    it('should return true if access token is invalid', () => {
      jest.spyOn(oAuthService, 'hasValidAccessToken').mockReturnValue(false);
      jest.spyOn(oAuthService, 'hasValidIdToken').mockReturnValue(true);

      TestBed.runInInjectionContext(() => {
        const result: boolean = service.hasNoValidToken();

        expect(result).toBeTruthy();
      });
    });

    it('should return true if id token is invalid', () => {
      jest.spyOn(oAuthService, 'hasValidAccessToken').mockReturnValue(true);
      jest.spyOn(oAuthService, 'hasValidIdToken').mockReturnValue(false);

      TestBed.runInInjectionContext(() => {
        const result: boolean = service.hasNoValidToken();

        expect(result).toBeTruthy();
      });
    });

    it('should return true if both tokens are invalid', () => {
      jest.spyOn(oAuthService, 'hasValidAccessToken').mockReturnValue(false);
      jest.spyOn(oAuthService, 'hasValidIdToken').mockReturnValue(false);

      TestBed.runInInjectionContext(() => {
        const result: boolean = service.hasNoValidToken();

        expect(result).toBeTruthy();
      });
    });
  });

  describe('logout', () => {
    it('should call oAuthService.logout', () => {
      const spy: jest.SpyInstance<void, any> = jest.spyOn(oAuthService, 'logOut');

      service.logout();

      expect(spy).toHaveBeenCalled();
    });
  });

  describe('logoutWithConfirmation', () => {
    it('should call sessionStorage.removeItem with id_token', () => {
      const spy: jest.SpyInstance<void> = jest.spyOn(Storage, 'removeIdTokenFromSessionStorage');

      service.logoutWithConfirmation();

      expect(spy).toHaveBeenCalled();
    });

    it('should call oAuthService.logout', () => {
      const customParameters: object = { client_id: environment.clientId };

      service.logoutWithConfirmation();

      expect(oAuthService.logOut).toHaveBeenCalledWith(customParameters);
    });
  });
});
