import { CommandOrder, CommandResource } from '@alfa-client/command-shared';
import { ApiError } from '@alfa-client/tech-shared';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Action } from '@ngrx/store';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createApiError } from 'libs/tech-shared/test/error';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { BescheidState, initialState, reducer } from './bescheid.reducer';

import * as CommandActions from '../../../../command-shared/src/lib/+state/command.actions';

describe('Bescheid Reducer', () => {
  describe('unknown action', () => {
    it('should return current state', () => {
      const action: Action = {} as Action;

      const result = reducer(initialState, action);

      expect(result).toBe(initialState);
    });
  });

  describe('bescheidCommand', () => {
    describe('on "createCommand" action', () => {
      it('should set resource loading', () => {
        const resource: VorgangWithEingangResource = createVorgangWithEingangResource();
        const action: Action = CommandActions.createCommand({
          resource,
          linkRel: VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT,
          command: { ...createCommandResource(), order: CommandOrder.CREATE_BESCHEID },
        });

        const state: BescheidState = reducer(initialState, action);

        expect(state.bescheidCommand.loading).toBeTruthy();
      });
    });

    describe('on "createCommandSuccess" action', () => {
      it('should set resource', () => {
        const command: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.CREATE_BESCHEID,
        };
        const action: Action = CommandActions.createCommandSuccess({ command });

        const state: BescheidState = reducer(initialState, action);

        expect(state.bescheidCommand.resource).toBe(command);
      });
    });

    describe('on "createCommandFailure" action', () => {
      const command: CommandResource = {
        ...createCommandResource(),
        order: CommandOrder.CREATE_BESCHEID,
      };

      it('should set error', () => {
        const apiError: ApiError = createApiError();
        const action: Action = CommandActions.createCommandFailure({
          command,
          error: { error: apiError },
        });

        const state: BescheidState = reducer(initialState, action);

        expect(state.bescheidCommand.error).toBe(apiError);
      });
    });
  });
});
