import {
  CollaborationListResource,
  OrganisationsEinheitResource,
} from '@alfa-client/collaboration-shared';
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, dispatchEventFromFixture, getMockComponent, mock } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { CollaborationService } from 'libs/collaboration-shared/src/lib/collaboration.service';
import { OrganisationsEinheitResourceSearchService } from 'libs/collaboration-shared/src/lib/organisations-einheit-resource-search.service';
import { createCollaborationListResource } from 'libs/collaboration-shared/test/collaboration';
import { createOrganisationsEinheitResource } from 'libs/collaboration-shared/test/organisations-einheit';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { CollaborationInVorgangContainerComponent } from './collaboration-in-vorgang-container.component';
import { CollaborationInVorgangComponent } from './collaboration-in-vorgang/collaboration-in-vorgang.component';

jest.mock('libs/collaboration-shared/src/lib/organisations-einheit-resource-search.service');

describe('CollaborationInVorgangContainerComponent', () => {
  let component: CollaborationInVorgangContainerComponent;
  let fixture: ComponentFixture<CollaborationInVorgangContainerComponent>;

  const collaborationInVorgangComp: string = getDataTestIdOf('collaboration-in-vorgang');

  const service: Mock<CollaborationService> = {
    ...mock(CollaborationService),
    isRequestFormVisible: jest.fn().mockReturnValue(of(false)),
  };

  const organisationsEinheitSearchService: Mock<OrganisationsEinheitResourceSearchService> = mock(
    OrganisationsEinheitResourceSearchService,
  );

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        CollaborationInVorgangContainerComponent,
        MockComponent(CollaborationInVorgangComponent),
      ],
      providers: [
        {
          provide: CollaborationService,
          useValue: service,
        },
        {
          provide: OrganisationsEinheitResourceSearchService,
          useValue: organisationsEinheitSearchService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(CollaborationInVorgangContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call service to get collaboration state list resource', () => {
      component.ngOnInit();

      expect(service.getList).toHaveBeenCalled();
    });

    it('should call service to get request form visbility', () => {
      component.ngOnInit();

      expect(service.isRequestFormVisible).toHaveBeenCalled();
    });

    it('should get selected result', () => {
      component.ngOnInit();

      expect(organisationsEinheitSearchService.getSelectedResult).toHaveBeenCalled();
    });
  });

  describe('collaboration in vorgang component', () => {
    const collaborationListResource: CollaborationListResource = createCollaborationListResource();
    const collaborationStateListResource: StateResource<CollaborationListResource> =
      createStateResource(collaborationListResource);

    describe('should be called with', () => {
      it('collaboration state list resource', () => {
        component.collaborationStateListResource$ = of(collaborationStateListResource);

        fixture.detectChanges();

        expect(getCollaborationInVorgangComponent().collaborationStateListResource).toBe(
          collaborationStateListResource,
        );
      });

      it('is request form visible', () => {
        component.isRequestFormVisible$ = of(true);

        fixture.detectChanges();

        expect(getCollaborationInVorgangComponent().isRequestFormVisible).toBeTruthy();
      });

      it('organisations einheit', () => {
        const organisationsEinheitResource: OrganisationsEinheitResource =
          createOrganisationsEinheitResource();
        component.selectedOrganisationsEinheit$ = of(organisationsEinheitResource);

        fixture.detectChanges();

        expect(getCollaborationInVorgangComponent().organisationsEinheit).toBe(
          organisationsEinheitResource,
        );
      });

      function getCollaborationInVorgangComponent(): CollaborationInVorgangComponent {
        return getMockComponent<CollaborationInVorgangComponent>(
          fixture,
          CollaborationInVorgangComponent,
        );
      }
    });

    it('should call hideRequestForm on output', () => {
      component.hideRequestForm = jest.fn();

      dispatchEventFromFixture(fixture, collaborationInVorgangComp, 'hideRequestForm');

      expect(component.hideRequestForm).toHaveBeenCalled();
    });

    it('should call showRequestForm on output', () => {
      component.showRequestForm = jest.fn();

      dispatchEventFromFixture(fixture, collaborationInVorgangComp, 'showRequestForm');

      expect(component.showRequestForm).toHaveBeenCalled();
    });
  });

  describe('show request form', () => {
    it('should call service', () => {
      component.showRequestForm();

      expect(service.showRequestForm).toHaveBeenCalled();
    });
  });

  describe('hide request form', () => {
    it('should call service', () => {
      component.hideRequestForm();

      expect(service.hideRequestForm).toHaveBeenCalled();
    });
  });

  describe('ngOnDestroy', () => {
    it('should call service to clear selected result', () => {
      component.ngOnDestroy();

      expect(organisationsEinheitSearchService.clearSelectedResult).toHaveBeenCalled();
    });
  });
});
