import {
  Collaboration,
  CollaborationListLinkRel,
  CollaborationListResource,
} from '@alfa-client/collaboration-shared';
import { HasLinkPipe, createStateResource } from '@alfa-client/tech-shared';
import {
  EventData,
  dispatchEventFromFixture,
  existsAsHtmlElement,
  getMockComponent,
  notExistsAsHtmlElement,
  triggerEvent,
} from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonComponent, OfficeIconComponent, UsersIconComponent } from '@ods/system';
import {
  createCollaboration,
  createCollaborationListResource,
} from 'libs/collaboration-shared/test/collaboration';
import { getDataTestIdAttributeOf, getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { CollaborationRequestFormComponent } from '../collaboration-request-form/collaboration-request-form.component';
import { OrganisationsEinheitComponent } from '../collaboration-request-form/organisations-einheit-container/organisations-einheit/organisations-einheit.component';
import { CollaborationInVorgangComponent } from './collaboration-in-vorgang.component';

describe('CollaborationInVorgangComponent', () => {
  let component: CollaborationInVorgangComponent;
  let fixture: ComponentFixture<CollaborationInVorgangComponent>;

  const anfrageErstellenButton: string = getDataTestIdAttributeOf('anfrage-erstellen-button');
  const collaborationRequestForm: string = getDataTestIdOf('collaboration-request-form');
  const collaborationRequestResult: string = getDataTestIdOf('collaboration-request-result');

  const collaborationListResource: CollaborationListResource = createCollaborationListResource();

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        CollaborationInVorgangComponent,
        HasLinkPipe,
        MockComponent(ButtonComponent),
        MockComponent(CollaborationRequestFormComponent),
        MockComponent(UsersIconComponent),
        MockComponent(OfficeIconComponent),
        MockComponent(OrganisationsEinheitComponent),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(CollaborationInVorgangComponent);
    component = fixture.componentInstance;
    component.collaborationStateListResource = createStateResource(collaborationListResource);
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('if create collaboration request link exists', () => {
    const collaborationListResourceWithLink: CollaborationListResource =
      createCollaborationListResource([CollaborationListLinkRel.CREATE_COLLABORATION_REQUEST]);
    beforeEach(() => {
      component.collaborationStateListResource = createStateResource(
        collaborationListResourceWithLink,
      );
      fixture.detectChanges();
    });

    describe('anfrage erstellen button', () => {
      describe('on request form visibility false', () => {
        beforeEach(() => {
          component.isRequestFormVisible = false;
        });

        it('should be shown', () => {
          fixture.detectChanges();

          existsAsHtmlElement(fixture, anfrageErstellenButton);
        });

        it('should call service on click', () => {
          fixture.detectChanges();
          const showRequestFormSpy: jest.SpyInstance = (component.showRequestForm.emit = jest.fn());

          dispatchEventFromFixture(fixture, anfrageErstellenButton, 'clickEmitter');

          expect(showRequestFormSpy).toHaveBeenCalled();
        });
      });

      it('should be hidden if request form visibility is true', () => {
        component.isRequestFormVisible = true;

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, anfrageErstellenButton);
      });
    });

    describe('zustaendige stelle', () => {
      describe('on request form visibility true', () => {
        beforeEach(() => {
          component.isRequestFormVisible = true;
        });
        it('should be shown', () => {
          fixture.detectChanges();

          existsAsHtmlElement(fixture, collaborationRequestForm);
        });

        describe('component', () => {
          it('should call service on hideFormular output', () => {
            fixture.detectChanges();
            const hideRequestFormSpy: jest.SpyInstance = (component.hideRequestForm.emit =
              jest.fn());

            dispatchEventFromFixture(fixture, collaborationRequestForm, 'hide');

            expect(hideRequestFormSpy).toHaveBeenCalled();
          });

          it('should call set collaboration on showResult output', () => {
            fixture.detectChanges();
            component.setCollaboration = jest.fn();
            const collaboration: Collaboration = createCollaboration();

            const eventData: EventData<CollaborationInVorgangComponent> = {
              fixture,
              elementSelector: collaborationRequestForm,
              name: 'showResult',
              data: collaboration,
            };
            triggerEvent(eventData);

            expect(component.setCollaboration).toHaveBeenCalledWith(collaboration);
          });

          it('should be called with', () => {
            fixture.detectChanges();

            const comp: CollaborationRequestFormComponent =
              getMockComponent<CollaborationRequestFormComponent>(
                fixture,
                CollaborationRequestFormComponent,
              );
            expect(comp.collaborationListResource).toBe(collaborationListResourceWithLink);
          });
        });
      });

      it('should be hidden if request form visibility is false', () => {
        component.isRequestFormVisible = false;

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, collaborationRequestForm);
      });
    });
  });

  describe('on existing collaboration', () => {
    it('should show result', () => {
      component.collaboration = createCollaboration();

      fixture.detectChanges();

      existsAsHtmlElement(fixture, collaborationRequestResult);
    });
  });

  describe('set collaboration', () => {
    const collaboration: Collaboration = createCollaboration();

    it('should set attribute', () => {
      component.setCollaboration(collaboration);

      expect(component.collaboration).toBe(collaboration);
    });

    it('should call emitter', () => {
      const hideRequestFormEmitSpy: jest.SpyInstance = (component.hideRequestForm.emit = jest.fn());

      component.setCollaboration(collaboration);

      expect(hideRequestFormEmitSpy).toBeCalled();
    });
  });
});
