import { getElementFromFixture } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import faker from '@faker-js/faker';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createInvalidParam } from 'libs/tech-shared/test/error';
import { CheckboxEditorComponent } from './checkbox-editor.component';

describe('CheckboxEditorComponent', () => {
  let component: CheckboxEditorComponent;
  let fixture: ComponentFixture<CheckboxEditorComponent>;

  const labelText: string = faker.word.noun();
  const errorMessageId: string = getDataTestIdOf(labelText + '-checkbox-editor-error');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [CheckboxEditorComponent],
    }).compileComponents();

    fixture = TestBed.createComponent(CheckboxEditorComponent);
    component = fixture.componentInstance;
    component.label = labelText;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('hasError', () => {
      it('should return true', () => {
        component.fieldControl.setErrors({ 1: createInvalidParam() });

        const result: boolean = component.hasError;

        expect(result).toBe(true);
      });

      it('should return false', () => {
        component.fieldControl.setErrors(null);

        const result: boolean = component.hasError;

        expect(result).toBe(false);
      });
    });
  });

  describe('template', () => {
    describe('error message', () => {
      it('should show', () => {
        component.fieldControl.setErrors({ 1: createInvalidParam() });
        fixture.detectChanges();

        const element: HTMLElement = getElementFromFixture(fixture, errorMessageId);

        expect(element).not.toBeEmptyDOMElement();
      });

      it('should hide', () => {
        const element: HTMLElement = getElementFromFixture(fixture, errorMessageId);

        expect(element).toBeEmptyDOMElement();
      });
    });
  });
});
