import { createInvalidParam, createProblemDetail } from '../../../test/error';
import { InvalidParam, ProblemDetail } from '../tech.model';
import { ValidationMessageCode } from '../validation/tech.validation.messages';
import * as TechValidationUtil from '../validation/tech.validation.util';
import { ConvertProblemDetailToErrorMessagesPipe } from './convert-problem-detail-to-error-messages.pipe';

describe('convertProblemDetailToErrorMessages', () => {
  const pipe = new ConvertProblemDetailToErrorMessagesPipe();

  it('create an instance', () => {
    expect(pipe).toBeTruthy();
  });

  describe('transform', () => {
    const getMessageForInvalidParam = jest.spyOn(TechValidationUtil, 'getMessageForInvalidParam');

    it('should not call getMessageForInvalidParam', () => {
      pipe.transform(null);

      expect(getMessageForInvalidParam).not.toHaveBeenCalled();
    });

    it('should call getMessageForInvalidParam', () => {
      pipe.transform(createProblemDetail());

      expect(getMessageForInvalidParam).toHaveBeenCalled();
    });

    it('should return array of error messages', () => {
      const expectedErrorMessage = 'Bitte  ausfüllen';
      const invalidParam: InvalidParam = {
        ...createInvalidParam(),
        reason: ValidationMessageCode.FIELD_EMPTY,
      };
      const problemDetail: ProblemDetail = createProblemDetail([invalidParam]);

      const errorMessages: string[] = pipe.transform(problemDetail);

      expect(errorMessages).toEqual([expectedErrorMessage]);
    });
  });
});
