import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { fakeAsync, tick } from '@angular/core/testing';
import { Resource } from '@ngxp/rest';
import { DummyLinkRel, DummyListLinkRel } from 'libs/tech-shared/test/dummy';
import { createDummyListResource, createDummyResource } from 'libs/tech-shared/test/resource';
import { BehaviorSubject, of, skip } from 'rxjs';
import { ResourceListService } from './list-resource.service';
import { ListResourceServiceConfig } from './resource.model';
import { ResourceRepository } from './resource.repository';
import {
  ListResource,
  StateResource,
  createEmptyStateResource,
  createStateResource,
} from './resource.util';

//Der Test muss nochmal ueberarbeitet werden - siehe resource.service.itcase.spec.ts
describe.skip('ResourceListService ITCase', () => {
  let service: ResourceListService<Resource, ListResource, Resource>;
  let config: ListResourceServiceConfig<Resource>;
  let resourceRepository: Mock<ResourceRepository>;

  const listLinkRel: string = DummyListLinkRel.LIST;
  const listResourceListLinkRel: string = DummyListLinkRel.LIST;
  const createLinkRel: string = DummyLinkRel.DUMMY;

  const baseResource: StateResource<Resource> = createStateResource(createDummyResource());

  const loadedListResource: ListResource = createDummyListResource();

  describe('getList', () => {
    const loadedListResource: ListResource = createDummyListResource();
    // *  - GIVEN listResource=leer, baseResource=leer WHEN baseResource neuer Eintrag THEN load ausführen AND wenn fertig listResource liefert Wert
    describe('with empty listResource and empty baseResource', () => {
      const baseResourceSubj: BehaviorSubject<StateResource<Resource>> = new BehaviorSubject<
        StateResource<Resource>
      >(createEmptyStateResource());

      beforeEach(() => {
        config = createConfigWithBaseResource(baseResourceSubj);
        initServiceAndMocksWithBaseResourceSubj(config);

        resourceRepository.getListResource.mockReturnValue(of(loadedListResource));
      });

      it('should load and return listResource on baseResource change', (done) => {
        let emitted: number = 0;
        service.getList().subscribe((listStateResource) => {
          emitted++;
          if (emitted == 1) {
            expect(listStateResource).toEqual(createEmptyStateResource());
          }
          if (emitted == 2) {
            expect(listStateResource.loading).toBeTruthy();
            expect(resourceRepository.getListResource).toHaveBeenCalled();
          }
          if (emitted === 3) {
            expect(listStateResource).toEqual(createStateResource(loadedListResource));
            done();
          }
        });
        baseResourceSubj.next(createStateResource(createDummyResource()));
      });
    });
    // *  - GIVEN listResource=befüllt, baseResource=befüllt WHEN subscribe THEN nicht laden, listResource liefert Wert
    describe('with filled listResource and filled baseResource', () => {
      const listStateResource: StateResource<ListResource> =
        createStateResource(createDummyListResource());

      const baseResourceSubj: BehaviorSubject<StateResource<Resource>> = new BehaviorSubject<
        StateResource<Resource>
      >(baseResource);

      beforeEach(() => {
        config = createConfigWithBaseResource(baseResourceSubj);
        initServiceAndMocksWithBaseResourceSubj(config);

        service.listResource.next(listStateResource);
      });

      it('should return listResource', (done) => {
        service.getList().subscribe((response) => {
          expect(response).toBe(listStateResource);
          done();
        });
      });

      it('should not call repository', fakeAsync(() => {
        service.getList().subscribe();
        tick();

        expect(resourceRepository.getListResource).not.toHaveBeenCalled();
      }));

      // *  - GIVEN listResource=befüllt, baseResource=befüllt WHEN baseResource wird leer THEN listResource leer
      describe('on baseResource changed to null', () => {
        it('should return empty state resource', (done) => {
          service
            .getList()
            .pipe(skip(1))
            .subscribe((listStateResource) => {
              expect(listStateResource).toEqual(createEmptyStateResource());
              done();
            });

          baseResourceSubj.next(createEmptyStateResource());
        });
      });

      // *  - GIVEN listResource=befüllt, baseResource=befüllt WHEN baseResource wechselt Wert THEN load ausführen AND wenn fertig listResource liefert neuen Wert
      describe('on baseResource changed', () => {
        it('should reloaded value', (done) => {
          let emitted: number = 0;
          service.getList().subscribe((response) => {
            emitted++;
            if (emitted === 1) {
              expect(response).toEqual(listStateResource);
            }
            if (emitted === 2) {
              expect(response.loading).toBeTruthy();
              expect(resourceRepository.getListResource).toHaveBeenCalled();
            }
            if (emitted === 3) {
              done();
            }
          });

          baseResourceSubj.next(createStateResource(createDummyResource()));
        });
      });
    });
  });

  function createConfigWithBaseResource(baseResource: BehaviorSubject<StateResource<Resource>>) {
    return {
      baseResource,
      listLinkRel,
      listResourceListLinkRel,
      createLinkRel,
    };
  }

  function initServiceAndMocksWithBaseResourceSubj(
    config: ListResourceServiceConfig<Resource>,
  ): void {
    resourceRepository = mock(ResourceRepository);
    service = new ResourceListService<Resource, ListResource, Resource>(
      config,
      useFromMock(resourceRepository),
    );

    resourceRepository.getListResource.mockReturnValue(of(loadedListResource));
  }
});
