import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { Store } from '@ngrx/store';
import { Subject } from 'rxjs';
import { UserProfileFacade } from './user-profile.facade';

import * as UserProfileActions from './user-profile.actions';
import * as UserProfileSelectors from './user-profile.selectors';

describe('UserProfileFacade', () => {
  let facade: UserProfileFacade;
  let store: Mock<Store>;

  let selectionSubject: Subject<any>;

  beforeEach(() => {
    store = mock(Store);

    selectionSubject = new Subject();

    store.select.mockReturnValue(selectionSubject);

    facade = new UserProfileFacade(useFromMock(<any>store));
  });

  it('is initialized', () => {
    expect(facade).toBeTruthy();
  });

  describe('getUserProfileSearchVisibility', () => {
    it('should select from store', () => {
      facade.getUserProfileSearchVisibility();

      expect(store.select).toHaveBeenCalledWith(UserProfileSelectors.userProfileSearchVisibilty);
    });
  });

  describe('showUserProfileSearch', () => {
    it('should dispatch action', () => {
      facade.showUserProfileSearch();

      expect(store.dispatch).toHaveBeenCalledWith(UserProfileActions.showUserProfileSearch());
    });
  });

  describe('hideUserProfileSearch', () => {
    it('should dispatch action', () => {
      facade.hideUserProfileSearch();

      expect(store.dispatch).toHaveBeenCalledWith(UserProfileActions.hideUserProfileSearch());
    });
  });
});
