import { BescheidService } from '@alfa-client/bescheid-shared';
import { CommandLinkRel, CommandResource } from '@alfa-client/command-shared';
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { mock } from '@alfa-client/test-utils';
import { OzgcloudDialogService, OzgcloudStrokedButtonWithSpinnerComponent } from '@alfa-client/ui';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { ComponentFixture, TestBed, fakeAsync, tick } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import {
  createCommandResource,
  createCommandStateResource,
} from 'libs/command-shared/test/command';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { BescheidenFormService } from '../../../bescheiden.formservice';
import { VorgangDetailBescheidenAbbrechenDialogComponent } from './vorgang-detail-bescheiden-abbrechen-dialog.component';

describe('VorgangDetailBescheidenAbbrechenDialogComponent', () => {
  let component: VorgangDetailBescheidenAbbrechenDialogComponent;
  let fixture: ComponentFixture<VorgangDetailBescheidenAbbrechenDialogComponent>;

  const bescheidService = mock(BescheidService);
  const ozgcloudDialogService = mock(OzgcloudDialogService);
  const formService = mock(BescheidenFormService);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        VorgangDetailBescheidenAbbrechenDialogComponent,
        MockComponent(OzgcloudStrokedButtonWithSpinnerComponent),
        MockComponent(MatIcon),
      ],
      providers: [
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
        {
          provide: OzgcloudDialogService,
          useValue: ozgcloudDialogService,
        },
        {
          provide: BescheidenFormService,
          useValue: formService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailBescheidenAbbrechenDialogComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('saveBescheidDraft', () => {
    beforeEach(() => {
      const submitCommand: StateResource<CommandResource> = createCommandStateResource([
        CommandLinkRel.EFFECTED_RESOURCE,
      ]);
      formService.submitDraft = jest.fn().mockReturnValue(of(submitCommand));
    });

    it('should call formService.submit', () => {
      component.saveBescheidDraft();

      expect(formService.submitDraft).toHaveBeenCalled();
    });

    describe('on successfully done command', () => {
      it('should call ozgcloudDialogService.closeAll', fakeAsync(() => {
        component.saveBescheidDraft();
        component.saveDraftInProgress$.subscribe();
        tick();

        expect(ozgcloudDialogService.closeAll).toHaveBeenCalled();
      }));

      it('should call bescheid service to refresh list', fakeAsync(() => {
        component.saveBescheidDraft();
        component.saveDraftInProgress$.subscribe();
        tick();

        expect(bescheidService.refreshList).toHaveBeenCalled();
      }));
    });
  });

  describe('onCancel', () => {
    it('should call deleteBescheidDraft if Vorgang has bescheidDraft', () => {
      bescheidService.existsBescheidDraft.mockReturnValue(true);
      component.deleteBescheidDraft = jest.fn();

      component.onCancel();

      expect(component.deleteBescheidDraft).toHaveBeenCalled();
    });

    it('should call ozgcloudDialogService.closeAll if Vorgang has no bescheidDraft', () => {
      bescheidService.existsBescheidDraft.mockReturnValue(false);

      component.onCancel();

      expect(ozgcloudDialogService.closeAll).toHaveBeenCalled();
    });
  });

  describe('deleteBescheidDraft', () => {
    beforeEach(() => {
      const vorgangWithBescheid: VorgangWithEingangResource = createVorgangWithEingangResource([
        VorgangWithEingangLinkRel.BESCHEID_DRAFT,
      ]);
      formService.getVorgangWithEingangResource = jest.fn().mockReturnValue(vorgangWithBescheid);

      const commandResource: CommandResource = createCommandResource([
        CommandLinkRel.EFFECTED_RESOURCE,
      ]);
      const commandStateResource: StateResource<CommandResource> =
        createStateResource(commandResource);

      bescheidService.bescheidVerwerfen = jest.fn().mockReturnValue(of(commandStateResource));
    });

    it('should call bescheidService.bescheidVerwerfen', fakeAsync(() => {
      component.deleteBescheidDraft();
      component.deleteBescheid$.subscribe();
      tick();

      expect(bescheidService.bescheidVerwerfen).toHaveBeenCalled();
    }));

    it('should call ozgcloudDialogService.closeAll', () => {
      component.deleteBescheidDraft();

      expect(ozgcloudDialogService.closeAll).toHaveBeenCalled();
    });
  });
});
