import { BescheidService } from '@alfa-client/bescheid-shared';
import { CommandResource, tapOnCommandSuccessfullyDone } from '@alfa-client/command-shared';
import { StateResource, isLoaded } from '@alfa-client/tech-shared';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { VorgangService } from '@alfa-client/vorgang-shared';
import { DialogRef } from '@angular/cdk/dialog';
import { Component } from '@angular/core';
import { Observable, filter, first, switchMap } from 'rxjs';

@Component({
  selector: 'alfa-vorgang-detail-bescheiden-ueberspringen-dialog',
  templateUrl: './vorgang-detail-bescheiden-ueberspringen-dialog.component.html',
})
export class VorgangDetailBescheidenUeberspringenDialogComponent {
  bescheiderstellungUeberspringen$: Observable<StateResource<CommandResource>>;

  constructor(
    private readonly dialogRef: DialogRef,
    private readonly bescheidService: BescheidService,
    private readonly ozgcloudDialogService: OzgcloudDialogService,
    private readonly vorgangService: VorgangService,
  ) {}

  public onClose(): void {
    this.dialogRef.close();
  }

  public onConfirm(): void {
    this.bescheiderstellungUeberspringen$ = this.vorgangService.getVorgangWithEingang().pipe(
      filter(isLoaded),
      first(),
      switchMap((stateResource) =>
        this.bescheidService.bescheidErstellungUeberspringen(stateResource.resource).pipe(
          tapOnCommandSuccessfullyDone(() => {
            this.ozgcloudDialogService.closeAll();
          }),
        ),
      ),
    );
  }

  public onCancel(): void {
    this.dialogRef.close();
  }
}
