/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Component, Input, OnChanges, SimpleChanges } from '@angular/core';
import { BinaryFileListLinkRel, BinaryFileResource } from '@alfa-client/binary-file-shared';
import { StateResource, getEmbeddedResources, isNotNil } from '@alfa-client/tech-shared';
import {
  WiedervorlageLinkRel,
  WiedervorlageListLinkRel,
  WiedervorlageListResource,
  WiedervorlageResource,
  WiedervorlageService,
} from '@alfa-client/wiedervorlage-shared';
import { hasLink } from '@ngxp/rest';
import { isNull } from 'lodash-es';
import { Observable, of } from 'rxjs';
import { map } from 'rxjs/operators';
import { WiedervorlageFormService } from './wiedervorlage.formservice';

//TODO Container Component zwischenschalten
@Component({
  selector: 'alfa-wiedervorlage-form',
  templateUrl: './wiedervorlage-form.component.html',
  styleUrls: ['./wiedervorlage-form.component.scss'],
})
export class WiedervorlageFormComponent implements OnChanges {
  @Input() wiedervorlageListStateResource: StateResource<WiedervorlageListResource>;
  @Input() wiedervorlage: WiedervorlageResource;

  attachments$: Observable<BinaryFileResource[]> = of([]);

  readonly formServiceClass = WiedervorlageFormService;
  readonly wiedervorlageListLinkrel = WiedervorlageListLinkRel;

  constructor(
    public formService: WiedervorlageFormService,
    private wiedervorlageService: WiedervorlageService,
  ) {}

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.wiedervorlage && isNotNil(this.wiedervorlage)) {
      this.updateAttachments();
      this.patchWiedervorlage();
    }
  }

  updateAttachments(): void {
    if (hasLink(this.wiedervorlage, WiedervorlageLinkRel.ATTACHMENTS)) {
      this.attachments$ = this.wiedervorlageService
        .getAttachmentList(this.wiedervorlage)
        .pipe(
          map((attachmentList) =>
            getEmbeddedResources<BinaryFileResource>(
              attachmentList,
              BinaryFileListLinkRel.FILE_LIST,
            ),
          ),
        );
    } else {
      this.attachments$ = of([]);
    }
  }
  patchWiedervorlage(): void {
    if (!isNull(this.wiedervorlage)) {
      this.formService.patch(this.wiedervorlage);
    }
  }
}
