/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { UntypedFormBuilder } from '@angular/forms';
import { createEmptyStateResource } from '@alfa-client/tech-shared';
import { mock, Mock, useFromMock } from '@alfa-client/test-utils';
import { WiedervorlageResource, WiedervorlageService } from '@alfa-client/wiedervorlage-shared';
import { createWiedervorlageResource } from 'libs/wiedervorlage-shared/test/wiedervorlage';
import { of } from 'rxjs';
import { WiedervorlageFormService } from './wiedervorlage.formservice';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('WiedervorlageFormService', () => {
  let formService: WiedervorlageFormService;
  let wiedervorlageService: Mock<WiedervorlageService>;

  const formBuilder: UntypedFormBuilder = new UntypedFormBuilder();
  const wiedervorlageResource: WiedervorlageResource = createWiedervorlageResource();

  beforeEach(() => {
    wiedervorlageService = mock(WiedervorlageService);

    formService = new WiedervorlageFormService(formBuilder, useFromMock(wiedervorlageService));
  });

  it('should create', () => {
    expect(formService).toBeTruthy();
  });

  it('should have default date in 14 days', () => {
    const date = new Date();
    date.setDate(date.getDate() + 14);

    expect(formService.form.controls[WiedervorlageFormService.FIELD_FRIST].value.getDate()).toBe(
      date.getDate(),
    );
  });

  describe('submit', () => {
    beforeEach(() => {
      wiedervorlageService.createWiedervorlage.mockReturnValue(of(createEmptyStateResource()));
      wiedervorlageService.saveWiedervorlage.mockReturnValue(of(createEmptyStateResource()));
    });

    it('should call createWiedervorlage', () => {
      formService.source = null;

      formService.submit();

      expect(wiedervorlageService.createWiedervorlage).toHaveBeenCalled();
    });

    it('should call saveWiedervorlage', () => {
      formService.source = wiedervorlageResource;

      formService.submit();

      expect(wiedervorlageService.saveWiedervorlage).toHaveBeenCalled();
    });
  });
});
