package de.ozgcloud.alfa.loeschanforderung;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Objects;
import java.util.Optional;
import java.util.function.Predicate;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.RepresentationModelProcessor;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.ModelBuilder;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItem;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandHelper;
import de.ozgcloud.alfa.common.command.CommandOrder;

@Component
class LoeschAnforderungCommandProcessor implements RepresentationModelProcessor<EntityModel<Command>> {

	@Autowired
	private LoeschAnforderungService service;

	static final LinkRelation REL_REVOKE = LinkRelation.of("revoke");

	private static final Predicate<Command> IS_VORGANG_ZUM_LOESCHEN_MARKIEREN_AND_REVOKEABLE = command -> command
			.getCommandOrder() == CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN && CommandHelper.IS_REVOKEABLE.test(command);
	static final Predicate<Command> IS_LOESCHEN_ANFORDERUNG_ZURUECKNEHMEN_AND_REVOKEABLE = command -> command
			.getCommandOrder() == CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN && CommandHelper.IS_REVOKEABLE.test(command);

	@Override
	public EntityModel<Command> process(EntityModel<Command> model) {
		var command = model.getContent();

		if (Objects.isNull(command)) {
			return model;
		}
		return ModelBuilder.fromModel(model)
				.ifMatch(IS_LOESCHEN_ANFORDERUNG_ZURUECKNEHMEN_AND_REVOKEABLE)
				.addLink(() -> linkTo(methodOn(LoeschAnforderungCommandController.class).revoke(command.getRelationId(), command.getId(), null))
						.withRel(REL_REVOKE))
				.ifMatch(IS_VORGANG_ZUM_LOESCHEN_MARKIEREN_AND_REVOKEABLE)
				.addLinkIfPresent(() -> buildLink(command))
				.buildModel();
	}

	Optional<Link> buildLink(Command command) {
		return service.findLoeschAnforderung(command.getVorgangId()).map(loeschAnforderung -> buildLink(loeschAnforderung, command.getId()));
	}

	Link buildLink(VorgangAttachedItem loeschAnforderung, String commandId) {
		return linkTo(methodOn(LoeschAnforderungCommandController.class).revoke(loeschAnforderung.getId(), commandId, null))
				.withRel(REL_REVOKE);
	}
}