package de.ozgcloud.alfa.postfach;

import java.util.Objects;
import java.util.Optional;

import org.springframework.cloud.endpoint.RefreshEndpoint;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.FeatureToggleProperties;
import de.ozgcloud.alfa.vorgang.ServiceKonto;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Service
class PostfachSettingsService {

	private PostfachConfigGroup postfachConfigGroup;

	private final PostfachMailRemoteService remoteService;
	private final RefreshEndpoint refreshEndpoint;
	private final PostfachProperties postfachProperties;
	private final FeatureToggleProperties featureToggleProperties;

	public boolean isPostfachConfigured() {
		if (Objects.isNull(postfachConfigGroup)) {
			postfachConfigGroup = remoteService.getPostfachConfig();
		}
		return postfachConfigGroup.isConfigured();
	}

	public PostfachSettings getPostfachSettings(VorgangWithEingang vorgang) {
		return PostfachSettings.builder()
				.features(Features.builder()
						.reply(isReplyToMessageAllowed(vorgang))
						.build())
				.settings(Settings.builder()
						.signatur(getSignatur())
						.build())
				.build();
	}

	String getSignatur() {
		refreshPostfachProperties();
		return postfachProperties.getSignatur();
	}

	void refreshPostfachProperties() {
		refreshEndpoint.refresh();
	}

	boolean isReplyToMessageAllowed(VorgangWithEingang vorgang) {
		return Optional.ofNullable(vorgang.getHeader().getServiceKonto())
				.map(ServiceKonto::getType)
				.map(this::isReplyAllowed)
				.orElse(false);
	}

	boolean isReplyAllowed(String serviceKontoType) {
		if (featureToggleProperties.isReplyAlwaysAllowed()) {
			return true;
		}

		if (!isPostfachConfigured()) {
			return false;
		}

		return postfachConfigGroup.getPostfachConfigs().stream()
				.filter(postfachConfig -> postfachConfig.getType().equalsIgnoreCase(serviceKontoType))
				.map(PostfachConfig::isReplyAllowed)
				.findFirst()
				.orElse(false);
	}
}
