/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandController;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.common.command.LegacyOrder;
import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.RequiredArgsConstructor;

@RestController
@RequestMapping(WiedervorlageCommandController.WIEDERVORLAGE_COMMANDS)
@RequiredArgsConstructor
public class WiedervorlageCommandController {

	static final String WIEDERVORLAGE_COMMANDS = "/api/wiedervorlages/{wiedervorlageId}/{wiedervorlageVersion}/commands";

	private final WiedervorlageService service;

	@PostMapping
	public ResponseEntity<Void> updateWiedervorlage(@RequestBody CreateCommand command, @PathVariable String wiedervorlageId,
			@PathVariable long wiedervorlageVersion) {
		var wiedervorlage = service.getById(wiedervorlageId);
		var createdCommand = createCommand(wiedervorlage, command);

		return ResponseEntity.created(linkTo(CommandController.class).slash(createdCommand.getId()).toUri()).build();
	}

	Command createCommand(Wiedervorlage wiedervorlage, CreateCommand command) {
		switch (command.getOrder()) {
		case LegacyOrder.WIEDERVORLAGE_ERLEDIGEN: {
			var changed = wiedervorlage.toBuilder().done(true).build();
			service.updateNextFrist(wiedervorlage.getVorgangId(), changed);
			return service.erledigen(changed);
		}
		case LegacyOrder.WIEDERVORLAGE_WIEDEREROEFFNEN: {
			var changed = wiedervorlage.toBuilder().done(false).build();
			service.updateNextFrist(wiedervorlage.getVorgangId(), changed);
			return service.wiedereroeffnen(changed);
		}
		case LegacyOrder.EDIT_WIEDERVORLAGE: {
			var changed = updateWiedervorlageByCommand(wiedervorlage, (Wiedervorlage) command.getBody());
			service.updateNextFrist(wiedervorlage.getVorgangId(), changed);
			return service.editWiedervorlage(changed, wiedervorlage.getId(), wiedervorlage.getVersion());
		}
		default:
			throw new TechnicalException("Unsupported order " + command.getOrder());
		}
	}

	Wiedervorlage updateWiedervorlageByCommand(Wiedervorlage wiedervorlage, Wiedervorlage requestBody) {
		return wiedervorlage.toBuilder()
				.clearAttachments().attachments(requestBody.getAttachments())
				.betreff(requestBody.getBetreff())
				.beschreibung(requestBody.getBeschreibung())
				.frist(requestBody.getFrist()).build();
	}

	@RestController
	@RequestMapping(WiedervorlageCommandByVorgangController.WIEDERVORLAGE_COMMANDS_BY_VORGANG)
	@RequiredArgsConstructor
	public static class WiedervorlageCommandByVorgangController {

		static final String WIEDERVORLAGE_COMMANDS_BY_VORGANG = "/api/vorgangs/{vorgangId}/wiedervorlageCommands";

		private final WiedervorlageService service;

		@PostMapping
		public ResponseEntity<Void> createWiedervorlage(@RequestBody CreateCommand command, @PathVariable String vorgangId) {
			var wiedervorlage = (Wiedervorlage) command.getBody();
			var createdCommand = service.createWiedervorlage(wiedervorlage, vorgangId);

			service.updateNextFrist(vorgangId, wiedervorlage);

			return ResponseEntity.created(linkTo(CommandController.class).slash(createdCommand.getId()).toUri()).build();
		}
	}
}