package de.ozgcloud.alfa;

import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.hamcrest.core.StringEndsWith;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;

@AutoConfigureMockMvc
@SpringBootTest
@WithMockUser
class RootControllerITCase {

	@MockBean
	private CurrentUserService currentUserService;

	@Autowired
	private MockMvc mockMvc;

	@BeforeEach
	void init() {
		when(currentUserService.getUser()).thenReturn(UserProfileTestFactory.create());
	}

	@Nested
	class TestProcess {

		@Test
		void shouldAddResourceLink() throws Exception {
			var response = doRequest();

			response.andExpect(jsonPath("$._links.resource.href").value(StringEndsWith.endsWith("/api/resources?uri={uri}")));
		}
	}

	private ResultActions doRequest() throws Exception {
		return mockMvc.perform(get(RootController.PATH)).andExpect(status().isOk());
	}
}
