package de.ozgcloud.alfa.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentMatcher;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcOrganisationsEinheitGetRequest;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcOrganisationsEinheitGetResponse;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcOrganisationsEinheitSearchRequest;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcOrganisationsEinheitSearchResponse;
import de.ozgcloud.zufi.grpc.organisationseinheit.OrganisationsEinheitServiceGrpc.OrganisationsEinheitServiceBlockingStub;

class OrganisationsEinheitRemoteServiceTest {

	@Spy
	@InjectMocks
	private OrganisationsEinheitRemoteService service;

	@Mock
	private OrganisationsEinheitServiceBlockingStub serviceStub;

	@Mock
	private OrganisationsEinheitHeaderMapper organisationsEinheitHeaderMapper;
	@Mock
	private OrganisationsEinheitMapper organisationsEinheitMapper;

	@Nested
	class TestSearch {

		private final String searchBy = LoremIpsum.getInstance().getWords(5);
		private final GrpcOrganisationsEinheitSearchResponse grpcOrganisationsEinheitSearchResponse = GrpcOrganisationsEinheitSearchResponseTestFactory
				.create();

		@BeforeEach
		void setUpMocks() {
			when(serviceStub.search(argThat(requestContainsSearchBy()))).thenReturn(grpcOrganisationsEinheitSearchResponse);
		}

		@Test
		void shouldCallGrpcStub() {
			callService();

			verify(serviceStub).search(argThat(requestContainsSearchBy()));
		}

		@Test
		void shouldCallMapper() {
			callService().toList();

			verify(organisationsEinheitHeaderMapper).fromGrpc(GrpcOrganisationsEinheitSearchResponseTestFactory.ORGANISATIONS_EINHEIT);
		}

		@Test
		void shouldReturnMappedOrganisationsEinheitHeader() {
			var organisationsEinheitHeader = OrganisationsEinheitHeaderTestFactory.create();
			when(organisationsEinheitHeaderMapper.fromGrpc(GrpcOrganisationsEinheitSearchResponseTestFactory.ORGANISATIONS_EINHEIT)).thenReturn(
					organisationsEinheitHeader);

			var organisationsEinheitHeaders = callService();

			assertThat(organisationsEinheitHeaders).containsExactly(organisationsEinheitHeader);
		}

		private Stream<OrganisationsEinheitHeader> callService() {
			return service.search(searchBy);
		}

		private ArgumentMatcher<GrpcOrganisationsEinheitSearchRequest> requestContainsSearchBy() {
			return request -> request.getSearchBy().equals(searchBy);
		}
	}

	@Nested
	class TestGetById {

		private final GrpcOrganisationsEinheitGetRequest request = GrpcOrganisationsEinheitGetRequestTestFactory.create();
		private final GrpcOrganisationsEinheitGetResponse response = GrpcOrganisationsEinheitGetResponseTestFactory.create();
		private final OrganisationsEinheit organisationsEinheit = OrganisationsEinheitTestFactory.create();

		@BeforeEach
		void setUp() {
			doReturn(request).when(service).buildGetByIdRequest(OrganisationsEinheitTestFactory.ID);
			when(serviceStub.getById(request)).thenReturn(response);
			doReturn(organisationsEinheit).when(service).getOrganisationsEinheitFromGetByIdResponse(response);
		}

		@Test
		void shouldBuildRequest() {
			callService();

			verify(service).buildGetByIdRequest(OrganisationsEinheitTestFactory.ID);
		}

		@Test
		void shouldCallGrpcStub() {
			callService();

			verify(serviceStub).getById(request);
		}

		@Test
		void shouldBuildResponse() {
			callService();

			verify(service).getOrganisationsEinheitFromGetByIdResponse(response);
		}

		@Test
		void shouldReturnOrganisationsEinheit() {
			var get = service.getById(OrganisationsEinheitTestFactory.ID);

			assertThat(get).isEqualTo(organisationsEinheit);
		}

		private OrganisationsEinheit callService() {
			return service.getById(OrganisationsEinheitTestFactory.ID);
		}
	}

	@Nested
	class TestBuildGetByIdRequest {

		@Test
		void shouldHaveId() {
			var request = service.buildGetByIdRequest(OrganisationsEinheitTestFactory.ID);

			assertThat(request.getId()).isEqualTo(OrganisationsEinheitTestFactory.ID);
		}
	}

	@Nested
	class TestGetOrganisationsEinheitFromGetByIdResponse {

		private final GrpcOrganisationsEinheitGetResponse response = GrpcOrganisationsEinheitGetResponseTestFactory.create();
		private final OrganisationsEinheit organisationsEinheit = OrganisationsEinheitTestFactory.create();

		@BeforeEach
		void setUp() {
			when(organisationsEinheitMapper.fromGrpc(response.getOrganisationsEinheit())).thenReturn(organisationsEinheit);
		}

		@Test
		void shouldCallMapper() {
			callService();

			verify(organisationsEinheitMapper).fromGrpc(response.getOrganisationsEinheit());
		}

		@Test
		void shouldReturnOrgnisationsEinheit() {
			var built = callService();

			assertThat(built).isEqualTo(organisationsEinheit);
		}

		private OrganisationsEinheit callService() {
			return service.getOrganisationsEinheitFromGetByIdResponse(response);
		}
	}
}
