/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.clientattribute;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.vorgang.grpc.clientAttribute.ClientAttributeServiceGrpc.ClientAttributeServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcUpdateClientAttributeRequest;

class ClientAttributeRemoteServiceTest {
	private final ClientAttribute clientAttribute = ClientAttribute.HAS_NEW_POSTFACH_NACHRICHT;
	@Spy
	@InjectMocks
	private ClientAttributeRemoteService service;

	@Mock
	private ClientAttributeServiceBlockingStub serviceStub;

	@Mock
	private GrpcUpdateClientAttributeRequest request;

	@Nested
	class TestSetBooleanReadOnlyClientAttribute {
		private final Boolean value = false;

		@BeforeEach
		void mockBuildRequest() {
			doReturn(request).when(service).buildRequest(VorgangHeaderTestFactory.ID, clientAttribute, value);
		}

		@Test
		void shouldCallServiceStub() {
			callService();

			verify(serviceStub).update(request);
		}

		@Test
		void shouldCatchFunctionalException() {
			when(serviceStub.update(any())).thenThrow(new RuntimeException());

			assertDoesNotThrow(() -> callService());
		}

		private void callService() {
			service.setBooleanReadOnlyClientAttribute(VorgangHeaderTestFactory.ID, clientAttribute, value);
		}
	}

	@Nested
	class TestBuildRequest {
		@Test
		void shouldHaveAttribute() {
			GrpcUpdateClientAttributeRequest request = callService();

			assertThat(request.getAttribute()).isNotNull();
		}

		@Test
		void shouldHaveVorgangId() {
			GrpcUpdateClientAttributeRequest request = callService();

			assertThat(request.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldHaveAttributeName() {
			GrpcUpdateClientAttributeRequest request = callService();

			assertThat(request.getAttribute().getAttributeName()).isEqualTo(clientAttribute.getAttributeName());
		}

		@Test
		void shouldHaveClientName() {
			GrpcUpdateClientAttributeRequest request = callService();

			assertThat(request.getAttribute().getClientName()).isEqualTo(clientAttribute.getClientName().toString());
		}

		@ParameterizedTest
		@ValueSource(booleans = { true, false })
		void shouldHaveAttributeValue(boolean value) {
			GrpcUpdateClientAttributeRequest request = callService(value);

			assertThat(request.getAttribute().getValue()).isNotNull();
			assertThat(request.getAttribute().getValue().getBoolValue()).isEqualTo(value);
		}

		private GrpcUpdateClientAttributeRequest callService() {
			return callService(false);
		}

		private GrpcUpdateClientAttributeRequest callService(boolean value) {
			return service.buildRequest(VorgangHeaderTestFactory.ID, clientAttribute, value);
		}
	}
}
