/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.historie;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.List;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class HistorieControllerTest {
	private final String PATH = HistorieController.PATH + "?vorgangId=";

	@Nested
	class TestGetHistorieList {
		@Spy
		@InjectMocks
		private HistorieController controller;

		@Mock
		private HistorieService service;

		@Mock
		private HistorieModelAssembler modelAssembler;

		private MockMvc mockMvc;

		@BeforeEach
		void init() {
			mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
			when(service.findFinishedCommands(any())).thenReturn(createCommandStream());
		}

		private Stream<Command> createCommandStream() {
			return List.of(CommandTestFactory.create()).stream();
		}

		@Test
		void shouldCallService() throws Exception {
			mockMvc.perform(get(PATH + VorgangHeaderTestFactory.ID)).andExpect(status().isOk());

			verify(service).findFinishedCommands(any());
		}

	}

}
