package de.ozgcloud.alfa.postfach;

import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;

import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileService;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.user.UserProfile;
import de.ozgcloud.alfa.common.user.UserService;
import de.ozgcloud.alfa.postfach.PostfachMailExportData.PostfachMailExportDataBuilder;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.xoev.xdomea.DokumentType;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
public class ExportNachrichtService {
	private final PostfachMailService postfachMailService;
	private final BinaryFileService binaryFileService;
	private final UserService userService;

	public PostfachMailExportData createExportData(VorgangWithEingang vorgang) {
		var builder = PostfachMailExportData.builder();
		getPostfachMails(vorgang.getId())
				.forEach(mail -> addPostfachMailExportData(
						new PostfachMailExportInput(mail, getOrganisationseinheitenId(vorgang), getAttachments(mail)), builder));
		return builder.build();
	}

	public Stream<PostfachMail> getPostfachMails(String vorgangId) {
		return postfachMailService.getAll(vorgangId);
	}

	private String getOrganisationseinheitenId(VorgangWithEingang vorgang) {
		return vorgang.getEingang().getZustaendigeStelle().getOrganisationseinheitenId();
	}

	List<OzgFile> getAttachments(PostfachMail postfachMail) {
		return binaryFileService.getFiles(postfachMail.getAttachments()).toList();
	}

	void addPostfachMailExportData(PostfachMailExportInput postfachMailExportInput, PostfachMailExportDataBuilder builder) {
		builder.attachments(postfachMailExportInput.attachments());
		builder.dokumentType(buildDokumentType(postfachMailExportInput));
	}

	DokumentType buildDokumentType(PostfachMailExportInput postfachMailExportInput) {
		return DokumentTypeBuilder.builder()
				.withPostfachMail(postfachMailExportInput.postfachMail())
				.withOzgFiles(postfachMailExportInput.attachments())
				.withUserProfile(getUserProfile(postfachMailExportInput.postfachMail()).orElse(null))
				.withOrganisationseinheitenId(postfachMailExportInput.organisationseinheitenId())
				.build();
	}

	Optional<UserProfile> getUserProfile(PostfachMail postfachMail) {
		return Optional.ofNullable(postfachMail.getCreatedBy()).map(userService::getById);
	}
}
