package de.ozgcloud.alfa.common;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.ZonedDateTime;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.export.IdentifikationObjektTypeTestFactory;
import de.xoev.xdomea.IdentifikationObjektType;
import de.xoev.xdomea.VersionType;

class AnlageDokumentTypeBuilderTest {

	private final OzgFile ozgFile = OzgFileTestFactory.create();
	private final static ZonedDateTime CREATED_AT = ZonedDateTime.now();
	@Spy
	private AnlageDokumentTypeBuilder builder = AnlageDokumentTypeBuilder.builder().withOzgFile(ozgFile).withCreatedAt(CREATED_AT);

	@Nested
	class TestBuild {
		private MockedStatic<IdentifikationObjektTypeBuilder> identifikationObjektTypeBuilderMockedStatic;
		@Mock
		private IdentifikationObjektTypeBuilder identifikationObjektTypeBuilder;
		private final IdentifikationObjektType identifikationObjektType = IdentifikationObjektTypeTestFactory.create();

		private MockedStatic<VersionTypeBuilder> versionTypeBuilderMockedStatic;
		@Mock
		private VersionTypeBuilder versionTypeBuilder;
		private final VersionType version = VersionTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			identifikationObjektTypeBuilderMockedStatic = mockStatic(IdentifikationObjektTypeBuilder.class);
			identifikationObjektTypeBuilderMockedStatic.when(IdentifikationObjektTypeBuilder::builder).thenReturn(identifikationObjektTypeBuilder);
			when(identifikationObjektTypeBuilder.withObjectID(OzgFileTestFactory.ID.toString())).thenReturn(identifikationObjektTypeBuilder);
			when(identifikationObjektTypeBuilder.build()).thenReturn(identifikationObjektType);

			versionTypeBuilderMockedStatic = mockStatic(VersionTypeBuilder.class);
			versionTypeBuilderMockedStatic.when(VersionTypeBuilder::builder).thenReturn(versionTypeBuilder);
			when(versionTypeBuilder.withCreatedAt(CREATED_AT)).thenReturn(versionTypeBuilder);
			when(versionTypeBuilder.withOzgFile(ozgFile)).thenReturn(versionTypeBuilder);
			when(versionTypeBuilder.build()).thenReturn(version);
		}

		@AfterEach
		void tearDown() {
			identifikationObjektTypeBuilderMockedStatic.close();
			versionTypeBuilderMockedStatic.close();
		}

		@Test
		void shouldSetObjectId() {
			builder.build();

			verify(identifikationObjektTypeBuilder).withObjectID(ozgFile.getId().toString());
		}

		@Test
		void shouldBuildIdentifikationObjectType() {
			builder.build();

			verify(identifikationObjektTypeBuilder).build();
		}

		@Test
		void shouldHaveIdentifikation() {
			var anlage = builder.build();

			assertThat(anlage.getIdentifikation()).isEqualTo(identifikationObjektType);
		}

		@Test
		void shouldSetCreatedAt() {
			builder.build();

			verify(versionTypeBuilder).withCreatedAt(CREATED_AT);
		}

		@Test
		void shouldSetOzgFile() {
			builder.build();

			verify(versionTypeBuilder).withOzgFile(ozgFile);
		}

		@Test
		void shouldBuildVersionType() {
			builder.build();

			verify(versionTypeBuilder).build();
		}

		@Test
		void shouldHaveVersion() {
			var anlage = builder.build();

			assertThat(anlage.getVersion()).contains(version);
		}
	}
}
